@@ -23,11 +23,11 @@ except AttributeError:
     JSONDecodeError = ValueError
 
 class Serial(IOHandler):
-    poll_frequency = 100
+    poll_frequency = 200
 
     @classmethod
     def available_hosts(cls):
-        devices = comports()
+        devices = comports(include_links=True)
 
         return [d.device for d in devices]
 
